<?php
/*
Plugin Name: Exploit Scanner
Plugin URI: http://ocaoimh.ie/exploit-scanner/
Description: Scans your WordPress site for possible exploits.
Version: 0.97.3
Author: Donncha O Caoimh
Author URI: http://ocaoimh.ie/
*/

@ini_set( 'max_execution_time', 120 );

function exploitscanner_menu() {
	$page_hook = add_management_page( 'Exploit Scanner', 'Exploit Scanner', 'manage_options', 'exploit-scanner', 'exploitscanner_admin_page' );
	
	$help = '<p><strong>Interpreting the Results</strong></p>
	<p>It is likely that this scanner will find false positives (i.e. files which do not contain malicious code). However, it is best to err
	on the side of caution; if you are unsure then ask in the <a href="http://wordpress.org/support/" target="_blank">Support Forums</a>,
	download a fresh copy of a plugin, search the Internet for similar situations, et cetera. You should be most concerned if the scanner is:
	making matches around unknown external links; finding base64 encoded text in modified core files or the <code>wp-config.php</code> file;
	listing extra admin accounts; or finding content in posts which you did not put there.</p>
	<p>Understanding the three different result levels:</p>
	<ul>
		<li><strong>Severe:</strong> results that are often strong indicators of a hack (though they are not definitive proof)</li>
		<li><strong>Warning:</strong> these results are more commonly found in innocent circumstances than Severe matches, but they should
			still be treated with caution</li>
		<li><strong>Note:</strong> lowest priority, showing results that are very commonly used in legitimate code or notifications about events
			such as skipped files</li>
	</ul>
	
	<p><strong>Help! I think I have been hacked!</strong></p>
	<p>Follow the guides from the Codex:</p>
	<ul>
		<li><a href="http://codex.wordpress.org/FAQ_My_site_was_hacked">Codex: FAQ - My site was hacked</a></li>
		<li><a href="http://codex.wordpress.org/Hardening_WordPress">Codex: Hardening WordPress</a></li>
	</ul>
	<p>Ensure that you change <strong>all</strong> of your WordPress related passwords (site, FTP, MySQL, etc.). A regular backup routine
	(either manual or plugin powered) is extremely useful; if you ever find that your site has been hacked you can easily restore your site from
	a clean backup and fresh set of files and, of course, use a new set of passwords.</p>';

	add_contextual_help( $page_hook, $help );
}
add_action( 'admin_menu', 'exploitscanner_menu' );

function exploitscanner_admin_page() {
	global $wp_version;
	
	// non-ajax scan form processing
	if ( isset($_POST['action']) && 'scan' == $_POST['action'] ) {
		check_admin_referer( 'exploitscanner-scan_all' );
		
		$fsl = ( ! isset($_POST['filesize_limit']) || ! is_numeric($_POST['filesize_limit']) ) ? 400 : (int) $_POST['filesize_limit'];
		$dis = ( isset($_POST['display_pattern']) ) ? true : false;
		
		$scanner = new File_Exploit_Scanner( ABSPATH, array( 'start' => 0, 'fsl' => $fsl, 'display_pattern' => $dis ) );
		$scanner->run();
		
		$scanner = new DB_Exploit_Scanner();
		$scanner->run();
	}
	
	delete_transient( 'exploitscanner_results_trans' );
	$results = get_option( 'exploitscanner_results' );
	?>
	<div class="wrap">
		<h2>Exploit Scanner</h2>
		
		<p>This script searches through your WordPress install for signs that may indicate that your website has been compromised by hackers. It does <strong>NOT</strong> remove anything, this is left for the user to do.</p>
		
		<form action="<?php admin_url( 'tools.php?page=exploit-scanner' ); ?>" method="post">
			<?php wp_nonce_field( 'exploitscanner-scan_all' ); ?>
			<input type="hidden" name="action" value="scan" />
			<table class="form-table">
				<tr>
					<th scope="row"><label for="display_pattern">Search for suspicious styles:</label></th>
					<td><input type="checkbox" id="display_pattern" name="display_pattern" checked="checked" value="1" /> <span class="description">(<code>display:none</code> and <code>visibility:hidden</code> can be used to hide spam, but may cause many false positives)</span></td>
				</tr>			
				<tr>
					<th scope="row"><label for="filesize_limit">Upper file size limit:</label></th>
					<td><input type="text" size="3" id="filesize_limit" name="filesize_limit" value="400" />KB <span class="description">(files larger than this are skipped and will be listed at the end of scan)</span></td>
				</tr>
				<tr class="hide-if-no-js">
					<th scope="row"><label for="max_test_files">Number of files per batch:</label></th>
					<td>
						<select id="max_test_files" name="max_test_files">
							<option value="100">100</option>
							<option value="150" selected="selected">150</option>
							<option value="250">250</option>
							<option value="500">500</option>
							<option value="1000">1000</option>
						</select>
						<span class="description">(to help reduce memory limit errors the scan processes a series of file batches)</span>
					</td>
				</tr>
			</table>
		
			<p class="submit"><input type="submit" id="run-scanner" class="button-primary" value="Run the Scan" /></p>
		</form>
		
		<script type="text/javascript">
			jQuery(document).ready(function($){
				$('#run-scanner').click( function() {
					var fsl = $('#filesize_limit').val(),
						max = parseInt( $('#max_test_files').val() ),
						dis = ($('#display_pattern:checked').val() !== undefined);
						
					$('#scan-results').hide();
					$('#scan-loader').show();				
					exploitscanner_file_scan(0, fsl, max, dis);
					return false;
				});
				
				$('#hide-skipped').toggle( function() {
					$('.skipped-file').hide();
					$(this).html('Show skipped files');
				}, function() {
					$('.skipped-file').show();
					$(this).html('Hide skipped files');
				});
			});
			
			var exploitscanner_file_scan = function(s, fsl, max, dis) {
				jQuery.post( ajaxurl, {
					action: 'exploit-scanner_file_scan',
					start: s,
					filesize_limit: fsl,
					max_batch_size: max,
					display_pattern: dis,
					_ajax_nonce: '<?php echo wp_create_nonce( 'exploit-scanner_scan' ); ?>'
				}, function(r) {
					if ( 'Complete' != r ) {
						jQuery('#scan-loader span').html(r);
						exploitscanner_file_scan(s+max, fsl, max, dis);						
					} else {
						exploitscanner_db_scan();
					}
				});
			};
			
			var exploitscanner_db_scan = function() {
				jQuery('#scan-loader span').html('Scanning database...');
				jQuery.post( ajaxurl, {
					action: 'exploit-scanner_db_scan',
					_ajax_nonce: '<?php echo wp_create_nonce( 'exploit-scanner_scan' ); ?>'
				}, function(r) {
					jQuery('#scan-loader img').hide();
					jQuery('#scan-loader span').html('Scan complete. Refresh the page to view the results.');
					window.location.reload(false);
				});
			};
		</script>
		
		<div id="scan-loader" style="display:none;margin:10px;padding:10px;background:#f7f7f7;border:1px solid #c6c6c6;text-align:center">
			<p><strong>Searching your filesystem and database for possible exploit code</strong></p>
			<p><span style="margin-right:5px">Files scanned: 0...</span><img src="<?php echo plugins_url( 'loader.gif', __FILE__ ); ?>" height="16px" width="16px" alt="loading-icon" /></p>
		</div>
		
		<div id="scan-results">
		<?php if ( ! $results ) : ?>
			<h3>Results</h3><p>No results stored.</p>
		<?php else : exploitscanner_show_results( $results ); endif; ?>
		</div>
		
		<h3>General Information</h3>
		<?php echo exploitscanner_list_admins(); ?>
		
		<h4>DISCLAIMER</h4>
		<p>Unfortunately it's impossible to catch every hack and it's all too easy to catch false positives (show a file as suspicious when in reality it is clean). If you have been hacked, this script may help you track down what files, comments or posts have been modified. On the other hand, if this script indicates your blog is clean, don't believe it. This is far from foolproof.</p>
		
		<p><strong>For the paranoid...</strong><br />
		To prevent someone hiding malicious code inside this plugin and to check that the signatures file hasn't been changed, here are the MD5 hashes of these files. Compare them with the references on the plugin homepage. You'll get extra points if you check this file has the actual md5_file() calls.</p>
		<p style="text-align: center">MD5 of exploit-scanner.php: <code><?php echo md5_file(__FILE__); ?></code></p>
		<?php if ( file_exists( dirname( __FILE__ ) . '/hashes-' . $wp_version . '.php' ) ) { ?>
			<p style="text-align: center">MD5 of hashes-<?php echo $wp_version; ?>.php: <code><?php echo md5_file( dirname( __FILE__ ) . '/hashes-' . $wp_version . '.php' ); ?></code></p>
		<?php } ?>
	</div>
<?php }

// display the tables of results
function exploitscanner_show_results( $results ) {
	if ( ! is_array($results) ) {
		echo 'Unfortunately the results appear to be malformed/corrupted. Try scanning again.';
		return;
	}

	$result = '<h3>Results</h3>';
	
	foreach ( array('severe','warning','note') as $l ) {
		if ( ! empty($results[$l]) ) {
			if ( $l == 'note' ) $result .= '<div style="float:right;font-size:11px;margin-top:1.3em"><a href="#" id="hide-skipped" class="hide-if-no-js">Hide skipped files</a></div>';
			$result .= '<h4>Level ' . ucwords($l) . ' (' . count($results[$l]) . ' matches)</h4>';
			$result .= '<table class="widefat fixed">
			<thead>
			<tr>
				<th scope="col" style="width:50%">Location / Description</th>
				<th scope="col">What was matched</th>
			</tr>
			</thead>
			<tbody>';
			
			foreach ( $results[$l] as $r )
				$result .= exploitscanner_draw_row( $r );
				
			$result .= '</tbody></table>';
		}
	}
	
	echo $result;
}

// draw a result row
function exploitscanner_draw_row( $r ) {
	$class = ( ! empty($r['class']) ) ? ' class="'.$r['class'].'"' : '';		

	$html = '<tr' . $class . '><td><strong>' . $r['loc'];
	
	if ( ! empty($r['line_no']) )
		$html .= ':' . $r['line_no'] . '</strong>';
	elseif ( ! empty($r['post_id']) )
		$html .= '</strong> <a href="' . get_edit_post_link($r['post_id']) . '" title="Edit this item">Edit</a>';
	elseif ( ! empty($r['comment_id']) )
		$html .= '</strong> <a href="' . admin_url( "comment.php?action=editcomment&amp;c={$r['comment_id']}" ) . '" title="Edit this comment">Edit</a>';
	else
		$html .= '</strong>';
		
	$html .= '<br />'.$r['desc'].'</td><td>';
	
	if ( ! empty($r['line']) )
		$html .= '<code>' . exploitscanner_hilight($r['line']) . '</code>';

	return $html . '</td></tr>';
}

// ajax initiated scan processing
function exploitscanner_ajax_file_scan() {
	check_ajax_referer( 'exploit-scanner_scan' );
	
	if ( ! isset($_POST['start']) )
		die('Error: start not set.');
	else
		$start = (int) $_POST['start'];
		
	$fsl = ( ! isset($_POST['filesize_limit']) || ! is_numeric($_POST['filesize_limit']) ) ? 400 : (int) $_POST['filesize_limit'];		
	$max = ( ! isset($_POST['max_batch_size']) || ! is_numeric($_POST['max_batch_size']) ) ? 100 : (int) $_POST['max_batch_size'];
	$display_pattern = ( $_POST['display_pattern'] != 'false' ) ? true : false;
		
	$args = compact( 'start', 'fsl', 'max', 'display_pattern' );
	
	$scanner = new File_Exploit_Scanner( ABSPATH, $args );
	if ( $scanner->run() )
		echo 'Complete';
	else
		echo 'Files scanned: ' . ($start+$max) . '...';
	
	exit;
}
add_action( 'wp_ajax_exploit-scanner_file_scan', 'exploitscanner_ajax_file_scan' );

// ajax initiated scan processing
function exploitscanner_ajax_db_scan() {
	check_ajax_referer( 'exploit-scanner_scan' );
	
	$scanner = new DB_Exploit_Scanner();
	$scanner->run();
	
	echo 'Done';
	
	exit;
}
add_action( 'wp_ajax_exploit-scanner_db_scan', 'exploitscanner_ajax_db_scan' );

function exploitscanner_activate() {
	// some DB clean up
	$count = get_option( 'exploit_scanner_result_count' );
	if ( $count ) {
		$opts = array('exploit_scanner_result_count','exploit_scanner_file_count','exploit_scanner_other','exploit_scanner_wp-admin','exploit_scanner_wp-content','exploit_scanner_wp-includes');
		foreach ( $opts as $opt )
			delete_option( $opt );
		
		for( $i = 0; $i < $count; $i++ )
			delete_option( 'exploit_scanner_results_' . $i );
	}
}
register_activation_hook( __FILE__, 'exploitscanner_activate' );

function exploitscanner_deactivate() {
	delete_option( 'exploitscanner_results' );
	delete_transient( 'exploitscanner_results_trans' );
	delete_transient( 'exploitscanner_files' );
}
register_deactivation_hook( __FILE__, 'exploitscanner_deactivate' );

// display a list of users with 'user_level' == 10
function exploitscanner_list_admins() {
	global $wpdb;
	
	if ( method_exists( $wpdb, 'get_blog_prefix' ) )
		$level_key = $wpdb->get_blog_prefix() . 'capabilities';
	else
		$level_key = $wpdb->prefix . 'capabilities';
	
	$user_ids = $wpdb->get_col( $wpdb->prepare("SELECT user_id FROM $wpdb->usermeta WHERE meta_key = %s AND meta_value LIKE '%s'", $level_key, '%administrator%') );

	ob_start();
	?>
	<h4>Users with admin privileges</h4>
	<table class="widefat">
		<thead>
			<tr>
				<th scope="col" style="width: 5%">ID</th>
				<th scope="col">Username</th>
				<th scope="col">Name</th>
				<th scope="col">Email</th>
			</tr>
		</thead>
		<tbody>
	<?php	
	foreach ( $user_ids as $id ) {
		$user = get_userdata( $id );
		echo '<tr><td>' . intval($user->ID) . '</td><td>' . esc_html($user->user_login) . '</td><td>';
		if ( isset( $user->last_name ) && isset( $user->first_name ) )
			echo esc_html($user->first_name) . ' ' . esc_html($user->last_name);
		echo '</td><td>' . esc_html($user->user_email) . '</td></tr>';
	} ?>
		</tbody>
	</table>
	<?php
	$admin_table = ob_get_clean();
	return $admin_table;
}

// highlight pattern matched code samples
function exploitscanner_hilight( $text ) {
	$start = strpos( $text, '$#$#' ) - 50;
	if ( $start < 0 ) $start = 0;
	$end = strrpos( $text, '#$#$' ) + 50;
	
	$text = substr( $text, $start, $end - $start + 1 );

	return str_replace( array('$#$#','#$#$'), array('<span style="background:#ff0">','</span>'), $text );
}

/**
 * Exploit Scanner base class. Scanners should extend this.
 */
class Exploit_Scanner {
	var $results;
	
	function add_result( $level, $info ) {
		$this->results[$level][] = $info;
	}
	
	function store_results( $done = false ) {
		$stored = get_transient( 'exploitscanner_results_trans' );
		
		if ( empty($this->results) ) {
			if ( $done )
				update_option( 'exploitscanner_results', $stored );
			return;
		}

		if ( $stored && is_array($stored) )
			$this->results = array_merge_recursive( $stored, $this->results );
	
		if ( $done ) {
			update_option( 'exploitscanner_results', $this->results );
			delete_transient( 'exploitscanner_results_trans' );
		} else {
			set_transient( 'exploitscanner_results_trans', $this->results );
		}
	}
}

/**
 * File Scanner. Scans all files in given path for suspicious text.
 */
class File_Exploit_Scanner extends Exploit_Scanner {
	var $path;
	var $start;
	var $filesize_limit;
	var $max_batch_size;
	var $paged = true;
	var $files = array();
	var $modified_files = array();
	var $skip;
	var $complete = false;
	var $suspicious_patterns = array(
		// Direct Database queries
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(ALTER)+\s+/msiU' => array( 'level' => 'severe', 'desc' => 'Possible database table alteration' ),
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(CREATE)+\s+/msiU' => array( 'level' => 'severe', 'desc' => 'Possible database table creation' ),
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(DROP)+\s+/msiU' => array( 'level' => 'severe', 'desc' => 'Possible database table deletion' ),
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(DELETE)+\s+(FROM)+\s+/msiU' => array( 'level' => 'warning', 'desc' => 'Direct database delete query' ),
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(UPDATE)+\s+/msiU' => array( 'level' => 'warning', 'desc' => 'Direct database update query' ),
		'/(\\\$wpdb->|mysql_|WP_Query)+.+(SELECT)+\s.+/msiU' => array( 'level' => 'note', 'desc' => 'Direct database select query' ),
		'/(^GLOBAL)(\\\$wpdb->|mysql_|WP_Query)+/msiU' => array( 'level' => 'note', 'desc' => 'Possible direct database query' ),

		// Output of variables
		'/(echo|print|\<\?\=)+.+(\\\$GLOBALS|\\\$_SERVER|\\\$_GET|\\\$_REQUEST|\\\$_POST)+/msiU' => array( 'level' => 'warning', 'desc' => 'Possible output of restricted variables' ),
		'/(echo|print|\<\?\=)+.+(get_search_query)+/msiU' => array( 'level' => 'warning', 'desc' => 'Output of search query' ),
		
		// Possible exploit patterns
		'/<iframe/msiU' => array( 'level' => 'warning', 'desc' => 'iframes are sometimes used to load unwanted adverts and code on your site' ),
		'/ShellBOT/msiU' => array( 'level' => 'severe', 'desc' => 'This may be a script used by hackers to get control of your server' ),
		'/uname -a/msiU' => array( 'level' => 'severe', 'desc' => 'Tells a hacker what operating system your server is running' ),
		'/YW55cmVzdWx0cy5uZXQ=/msiU' => array( 'level' => 'severe', 'desc' => 'base64 encoded text found in Search Engine Redirect hack <a href="http://blogbuildingu.com/wordpress/wordpress-search-engine-redirect-hack">[1]</a>' ),
		'/eval\s*\(/msiU' => array( 'level' => 'severe', 'desc' => 'Often used to execute malicious code' ),
		'/String\.fromCharCode/msiU' => array( 'level' => 'warning', 'desc' => 'JavaScript sometimes used to hide suspicious code' ),
		'/\$_COOKIE\[\'yahg\'\]/msiU' => array( 'level' => 'severe', 'desc' => 'YAHG Googlerank.info exploit code <a href="http://creativebriefing.com/wordpress-hacked-googlerankinfo/">[1]</a>' ),
		'/ekibastos/msiU' => array( 'level' => 'severe', 'desc' => 'Possible Ekibastos attack <a href="http://ocaoimh.ie/did-your-wordpress-site-get-hacked/">[1]</a>' ),
		'/base64_decode\s*\(/msiU' => array( 'level' => 'severe', 'desc' => 'Used by malicious scripts to decode previously obscured data/programs' ),
		'/<!--[A-Za-z0-9]+--><\?php/msiU' => array( 'level' => 'severe', 'desc' => 'Symptom of a link injection attack <a href="http://www.kyle-brady.com/2009/11/07/wordpress-mediatemple-and-an-injection-attack/">[1]</a>' ),
		'/<script>\/\*(GNU GPL|LGPL)\*\/ try\{window.onload.+catch\(e\) \{\}<\/script>/msiU' => array( 'level' => 'severe', 'desc' => 'Possible "Gumblar" JavaScript attack <a href="http://threatinfo.trendmicro.com/vinfo/articles/securityarticles.asp?xmlfile=042710-GUMBLAR.xml">[1]</a> <a href="http://justcoded.com/article/gumblar-family-virus-removal-tool/">[2]</a>' ),
		'/php \$[a-zA-Z]*=\'as\';/msiU' => array( 'level' => 'severe', 'desc' => 'Symptom of the "Pharma Hack" <a href="http://blog.sucuri.net/2010/07/understanding-and-cleaning-the-pharma-hack-on-wordpress.html">[1]</a>' ),
		'/defined?\(\'wp_class_support/' => array( 'level' => 'severe', 'desc' => 'Symptom of the "Pharma Hack" <a href="http://blog.sucuri.net/2010/07/understanding-and-cleaning-the-pharma-hack-on-wordpress.html">[1]</a>' ),
	);
	

	function File_Exploit_Scanner( $path, $args ) {
		$this->__construct( $path, $args );
	}
	
	function __construct( $path, $args ) {
		$this->path = $path;
		
		if ( ! empty($args['max']) )
			$this->max_batch_size = $args['max'];
		else
			$this->page = false;
			
		if ( $args['display_pattern'] ) {
			$this->suspicious_patterns['/visibility: ?hidden/msiU'] = array( 'level' => 'note', 'desc' => 'CSS style used to hide parts of a web page (often used legitimately)' );
			$this->suspicious_patterns['/display: ?none/msiU'] = array( 'level' => 'note', 'desc' => 'CSS style used to hide parts of a web page (often used legitimately)' );	
		}

		$this->start = $args['start'];
		$this->filesize_limit = $args['fsl'];
		
		$this->skip = ltrim( str_replace( array( untrailingslashit( ABSPATH ), '\\' ), array( '', '/' ), __FILE__ ), '/' );
	}
	
	function run() {
		$this->get_files( $this->start );
		$this->file_pattern_scan();
		$this->store_results();
		return $this->complete;
	}
	
	function get_files( $s ) {
		global $wp_version;
		
		if ( 0 == $s ) {
			unset( $filehashes );
			$hashes = dirname(__FILE__) . '/hashes-'. $wp_version .'.php';
			if ( file_exists( $hashes ) )
				include_once( $hashes );
			else
				$this->add_result( 'severe', array(
					'loc' => 'hashes-'. $wp_version .'.php missing',
					'desc' => 'The file containing hashes of all WordPress core files appears to be missing; modified core files will no longer be detected and a lot more suspicious strings will be detected'
				) );
		
			$this->recurse_directory( $this->path );

			foreach( $this->files as $k => $file ) {
				// don't scan unmodified core files
				if ( isset( $filehashes[$file] ) ) {
					if ( $filehashes[$file] == md5_file( $this->path.'/'.$file ) ) {
						unset( $this->files[$k] );
						continue;
					} else {
						$this->add_result( 'warning', array(
							'loc' => $file,
							'desc' => 'Modified core file'
						) );
					}
				}
				
				// don't scan files larger than given limit
				if ( filesize($this->path . $file) > ($this->filesize_limit * 1024) ) {
					unset( $this->files[$k] );
					$this->add_result( 'note', array(
						'loc' => $file,
						'desc' => 'File skipped due to size',
						'class' => 'skipped-file'
					) );
				}
			}
			
			$this->files = array_values( $this->files );
			set_transient( 'exploitscanner_files', $this->files );
		} else {
			$this->files = get_transient( 'exploitscanner_files' );
		}
		
		// use files list to get a batch if paged
		if ( $this->paged && (count($this->files) - $s) > $this->max_batch_size ) {
			$this->files = array_slice( $this->files, $s, $this->max_batch_size );
		} else {
			$this->files = array_slice( $this->files, $s );
			$this->complete = true;
		}
	}
	
	function recurse_directory( $dir ) {
		if ( $handle = @opendir( $dir ) ) {
			while ( false !== ( $file = readdir( $handle ) ) ) {
				if ( $file != '.' && $file != '..' ) {
					$file = $dir . '/' . $file;
					if ( is_dir( $file ) ) {
						$this->recurse_directory( $file );
					} elseif ( is_file( $file ) ) {
						$this->files[] = str_replace( $this->path.'/', '', $file );
					}
				}
			}
			closedir( $handle );
		}
	}
	
	function file_pattern_scan() {
		foreach ( $this->files as $file ) {
			if ( $file != $this->skip ) {
				$contents = file( $this->path . $file );
				foreach ( $contents as $n => $line ) {
					foreach ( $this->suspicious_patterns as $pattern => $p ) {
						$test = preg_replace_callback( $pattern, array( &$this, 'replace' ), $line );
						if ( $line !== $test )
							$this->add_result( $p['level'], array(
								'loc' => $file,
								'line' => esc_html( $test ),
								'line_no' => $n+1,
								'desc' => $p['desc']						
							) );
					}
				}
			}
		}
	}
	
	function replace( $matches ) {
		return '$#$#' . $matches[0] . '#$#$';
	}
}

/**
 * Database Scanner. Scans WordPress database for suspicious post/comment text and plugins.
 */
class DB_Exploit_Scanner extends Exploit_Scanner {
	var $suspicious_text = array(
		'eval(' => array( 'level' => 'severe', 'desc' => 'Often used by hackers to execute malicious code' ),
		'<script' => array( 'level' => 'severe', 'desc' => 'JavaScript hidden in the database is normally a sign of a hack' ),
	);

	var $suspicious_post_text = array(
		'<iframe' => array( 'level' => 'warning', 'desc' => 'iframes are sometimes used to load unwanted adverts and code on your site' ),
		'<noscript' => array( 'level' => 'warning', 'desc' => 'Could be used to hide spam in posts/comments' ),
		'display:' => array( 'level' => 'warning', 'desc' => 'Could be used to hide spam in posts/comments' ),
		'visibility:' => array( 'level' => 'warning', 'desc' => 'Could be used to hide spam in posts/comments' ),
		'<script src' => array( 'level' => 'severe', 'desc' => 'Malicious scripts loaded in posts by hackers perform redirects, inject spam, etc.' ),
	);
	
	function DB_Exploit_Scanner() {
		$this->__construct();
	}
	
	function __construct() {}

	function run() {
		$this->scan_posts();
		$this->scan_plugins();
		$this->store_results(true);
	}
	
	function replace( $content, $text ) {
		$s = strpos( $content, $text ) - 25;
		if ( $s < 0 ) $s = 0;
		
		$content = preg_replace( '/('.$text.')/', '$#$#\1#$#$', $content );
		$content = substr( $content, $s, 150 );
		return $content;
	}
	
	function scan_posts() {
		global $wpdb;
	
		foreach ( $this->suspicious_post_text as $text => $info ) {
			$posts = $wpdb->get_results( "SELECT ID, post_title, post_content FROM {$wpdb->posts} WHERE post_type<>'revision' AND post_content LIKE '%{$text}%'" );
			if ( $posts )
				foreach ( $posts as $post ) {
					$content = $this->replace( $post->post_content, $text );
				
					$this->add_result( $info['level'], array(
						'loc' => 'Post: ' . esc_html($post->post_title),
						'line' => esc_html($content),
						'post_id' => $post->ID,
						'desc' => $info['desc']
					) );
				}

			$comments = $wpdb->get_results( "SELECT comment_ID, comment_author, comment_content FROM {$wpdb->comments} WHERE comment_content LIKE '%{$text}%'" );
			if ( $comments )
				foreach ( $comments as $comment ) {
					$content = $this->replace( $comment->comment_content, $text );
				
					$this->add_result( $info['level'], array(
						'loc' => 'Comment by ' . esc_html($comment->comment_author),
						'line' => esc_html($content),
						'comment_id' => $comment->comment_ID,
						'desc' => $info['desc']
					) );
				}
		}
	}
	
	function scan_plugins() {
		$active_plugins = get_option( 'active_plugins' );
		if ( ! empty( $active_plugins ) && is_array( $active_plugins ) ) {
			foreach ( $active_plugins as $plugin ) {
				if ( strpos( $plugin, '..' ) !== false || substr( $plugin, -4 ) != '.php' ) {
					if ( $plugin == '' )
						$desc = 'Blank entry found. Should be removed. It will look like \'i:0;s:0:\"\";\' in the active_records field.';
					else
						$desc = 'Active plugin with a suspicious name.';
						
					$this->add_result( 'severe', array(
						'loc' => 'Plugin: ' . esc_html( $plugin ),
						'desc' => $desc
					) );
				}
			}
		}
	}
}
